/*---------------------------------------------------------------------------------------------------------------------------------
   EPA - Region II, ESD
  ------------------------------------------------------------------------------
   Program: kendl()
   Purpose: 
  ------------------------------------------------------------------------------
   Usage:   see function
  ------------------------------------------------------------------------------
   Calls:   nothing
   Calling: general function
  ------------------------------------------------------------------------------
   Notes:   statistical functions some simple and some derived from:
William H. Press, Brian P. Flannery, Saul A. Teukolsky and William T. Vettering
    _NUMERICAL RECIPIES in C: The Art of Scientific Computing_
    Cambridge University Press, 1988
            This book is useful, but contains problems--most notably incorrect
	    subscripting of arrays (C is 0 to N-1; they use 1 to N).
  ------------------------------------------------------------------------------
   History:
     Author/Site:    Date:       Event:
   Alan Brenner      Jan 94      original coding
     alan_brenner@unixmail.edi.epa.gov
   Alan Brenner      Jan 95      added case based statistics
                                 added debuging
  ------------------------------------------------------------------------------
   Disclaimer:
     Although this program has been used by the U. S. Environmental Protection
   Agency, no warranty, expressed or implied, is made by the EPA as to the
   accurarcy and functioning of the program and related program material nor
   shall the fact of distribution constitute any such warranty, and no
   responsibility is assumed by the EPA in connection herewith.
---------------------------------------------------------------------------------------------------------------------------------*/

#include "stats.h"

double kendl(n, data1, data2, z, prob)
  long n;
  double *data1, *data2, *z, *prob;
{
     long n1=0,n2=0,j,k,is=0;
     double svar,aa,a1,a2,tau;

     if (debug) (void) fprintf(stderr,"kendl(%d,%p,%p,%p,%p)\n",n,data1,data2,z,prob);

     for (j=0; j<n-1; j++)
     {
	  for (k=j+1; k<n; k++)
	  {
	       a1=data1[j]-data1[k];
	       a2=data2[j]-data2[k];
	       aa=a1*a2;
	       if (aa)
	       {
		    n1++;
		    n2++;
		    aa>0.0 ? ++is : --is;
	       }
	       else
	       {
		    if (a1) ++n1;
		    if (a2) ++n2;
	       }
	  }
     }
     tau=((double) is)/(sqrt((double) n1)*sqrt((double) n2));
     svar=(4.0*n+10.0)/(9.0*n*(n-1.0));
     *z=tau/sqrt(svar);
     *prob=erfcc(fabs(*z)/sqrt(2.0));
     return(tau);
}
