/*---------------------------------------------------------------------------------------------------------------------------------
   EPA - Region II, ESD
  ------------------------------------------------------------------------------
   Program: linearreg1()
   Purpose: find linear regression given two variables
  ------------------------------------------------------------------------------
   Usage:   see function
  ------------------------------------------------------------------------------
   Calls:   nothing
   Calling: general function
  ------------------------------------------------------------------------------
   Notes:   statistical functions some simple and some derived from:
William H. Press, Brian P. Flannery, Saul A. Teukolsky and William T. Vettering
    _NUMERICAL RECIPIES in C: The Art of Scientific Computing_
    Cambridge University Press, 1988
            This book is useful, but contains problems--most notably incorrect
	    subscripting of arrays (C is 0 to N-1; they use 1 to N).
  ------------------------------------------------------------------------------
   History:
     Author/Site:    Date:       Event:
   Alan Brenner    1 Aug 95    Original coding
     alan_brenner@unixmail.edi.epa.gov
  ------------------------------------------------------------------------------
   Disclaimer:
     Although this program has been used by the U. S. Environmental Protection
   Agency, no warranty, expressed or implied, is made by the EPA as to the
   accurarcy and functioning of the program and related program material nor
   shall the fact of distribution constitute any such warranty, and no
   responsibility is assumed by the EPA in connection herewith.
---------------------------------------------------------------------------------------------------------------------------------*/

#include "stats.h"

#define TINY 1.0e-20

double linearreg1(ndata,xarray,yarray,slope,intercept,sigm,sigb)
  long ndata;						    /* number of items in xarray and yarray */
  double *xarray;					    /* array of x values */
  double *yarray;					    /* array of y values */
  double *slope;					    /* output line slope */
  double *intercept;					    /* output line y intercept */
  double *sigm;						    /* output probable uncertainty of slope */
  double *sigb;						    /* output probable uncertainty of intercept */
{
     long count;					    /* for loop counter */
     double temp;					    /* temp value */
     double sumtemp2=0.0;				    /* summation of (temp value) squared */
     double sumx=0.0;					    /* summation of xarray values */
     double sumy=0.0;					    /* summation of yarray values */
     double sumxmean;					    /* mean of sx */
     double chi2=0.0;					    /* chi-squared return value */

     if (debug) (void) fprintf(stderr,"linearreg1(%d,%p,%p,%p,%p,%p,%p)\n",ndata,xarray,yarray,slope,intercept,sigm,sigb);

     for (count=0; count<ndata; count++)		    /* calculate summations */
     {
	  sumx+=xarray[count];
	  sumy+=yarray[count];
     }
     sumxmean=sumx/ndata;				    /* mean of sumx */

     for (*intercept=0.0, count=0; count<ndata; count++)
     {
	  temp=xarray[count]-sumxmean;			    /* difference from mean */
	  sumtemp2+=temp*temp;				    /* sum of squared differences from the mean */
	  *intercept+=temp*yarray[count];
     }

     *intercept/=sumtemp2;				    /* intercept */
     *slope=(sumy-sumx*(*intercept))/ndata;		    /* slope */

     for (count=0; count<ndata; count++)
     {
	  temp=yarray[count]-*slope-*intercept*xarray[count];
	  chi2+=temp*temp;				    /* chisquared */
     }

     *sigm=sqrt((1.0+sumx*sumx/(ss*sumtemp2))/ss)*sqrt(*chi2/(ndata-2));
     *sigb=sqrt(1.0/sumtemp2)*sqrt(*chi2/(ndata-2));	    /* probable uncertainties */

     return(chi2);					    /* return chisquared */
}
