/*---------------------------------------------------------------------------------------------------------------------------------
   EPA - Region II, ESD
  ------------------------------------------------------------------------------
   Program: maxrc()
   Purpose: find maximum data value for each case and record its record number
  ------------------------------------------------------------------------------
   Usage:   see function
  ------------------------------------------------------------------------------
   Calls:   nothing
   Calling: general function
  ------------------------------------------------------------------------------
   Notes:   statistical functions some simple and some derived from:
William H. Press, Brian P. Flannery, Saul A. Teukolsky and William T. Vettering
    _NUMERICAL RECIPIES in C: The Art of Scientific Computing_
    Cambridge University Press, 1988
            This book is useful, but contains problems--most notably incorrect
	    subscripting of arrays (C is 0 to N-1; they use 1 to N).
  ------------------------------------------------------------------------------
   History:
     Author/Site:    Date:       Event:
   Alan Brenner      Jan 94      original coding
     alan_brenner@unixmail.edi.epa.gov
   Alan Brenner      Jan 95      added case based statistics
                                 added debuging
  ------------------------------------------------------------------------------
   Disclaimer:
     Although this program has been used by the U. S. Environmental Protection
   Agency, no warranty, expressed or implied, is made by the EPA as to the
   accurarcy and functioning of the program and related program material nor
   shall the fact of distribution constitute any such warranty, and no
   responsibility is assumed by the EPA in connection herewith.
---------------------------------------------------------------------------------------------------------------------------------*/

#include "stats.h"

void maxrc(rows, data, cases, case_index, output, recs)
  long rows;						    /* number of data items in data[], and case_index[] */
  long cases;						    /* number of unique values in case_index[] */
  long *case_index;					    /* index of data that holds which case each data value goes with */
  long **recs;						    /* address of an array for record number of maximums */
  double *data;						    /* the actual data */
  double **output;					    /* address of an array for maximums */
{
     long count;					    /* for loop counter */

     if (debug)
     {
	  (void) fprintf(stderr,"maxrc(%d,%p,%d,%p,%p,%p)\n",rows,data,cases,case_index,*output,*recs);
	  (void) fprintf(stderr,"initializing output\n");
     }

     for (count=0; count<rows; count++)			    /* initialize output with something valid for comparison */
     {
	  *(*output+case_index[count])=data[count];	    /* index is paired to data and holds the correct offset of output */
	  *(*recs+case_index[count])=count+1;
	  if (debug>7)
	  {
	       (void) fprintf(stderr,"offset %d\t:\t",case_index[count]);
	       (void) fprintf(stderr,"%f = %d\n",*(*output+case_index[count]),*(*recs+case_index[count]));
	  }
     }	       

     if (debug) (void) fprintf(stderr,"finding case maximums\n");
     for (count=0; count<rows; count++)			    /* for every data item */
     {
	  if (*(*output+case_index[count])<data[count])	    /* if the current maximum value for this data item's case */
	  {						    /* is smaller than the current maximum value */
	       *(*output+case_index[count])=data[count];    /* save the larger value */
	       *(*recs+case_index[count])=count+1;	    /* and the record number that it occurs on */
	       if (debug>6) (void) fprintf(stderr,"%10.0 is now %f.\n",count+1,*(*output+case_index[count]));
	  }
     }
}
