/*---------------------------------------------------------------------------------------------------------------------------------
   EPA - Region II, ESD
  ------------------------------------------------------------------------------
   Program: meanc()
   Purpose: find mean data value for each case
  ------------------------------------------------------------------------------
   Usage:   see function
  ------------------------------------------------------------------------------
   Calls:   nothing
   Calling: general function
  ------------------------------------------------------------------------------
   Notes:   statistical functions some simple and some derived from:
William H. Press, Brian P. Flannery, Saul A. Teukolsky and William T. Vettering
    _NUMERICAL RECIPIES in C: The Art of Scientific Computing_
    Cambridge University Press, 1988
            This book is useful, but contains problems--most notably incorrect
	    subscripting of arrays (C is 0 to N-1; they use 1 to N).
  ------------------------------------------------------------------------------
   History:
     Author/Site:    Date:       Event:
   Alan Brenner      Jan 94      original coding
     alan_brenner@unixmail.edi.epa.gov
   Alan Brenner      Jan 95      added case based statistics
                                 added debuging
  ------------------------------------------------------------------------------
   Disclaimer:
     Although this program has been used by the U. S. Environmental Protection
   Agency, no warranty, expressed or implied, is made by the EPA as to the
   accurarcy and functioning of the program and related program material nor
   shall the fact of distribution constitute any such warranty, and no
   responsibility is assumed by the EPA in connection herewith.
---------------------------------------------------------------------------------------------------------------------------------*/

#include "stats.h"

void meanc(rows, data, cases, case_index, hits, output)
  long rows;						    /* number of data items in data[], and case_index[] */
  long cases;						    /* number of unique values in case_index[], and items in hits[] */
  long *case_index;					    /* index of data that holds which case each data value goes with */
  long *hits;						    /* records number of occurances in each case_index[] case */
  double *data;						    /* the actual data */
  double **output;					    /* address of an array for minimums */
{
     long count;					    /* for loop counter */

     if (debug) (void) fprintf(stderr,"meanc(%d,%p,%d,%p,%p,%p)\n",rows,data,cases,case_index,hits,output);

     for (count=0; count<cases; count++)		    /* initialize output with 0 */
	  *(*output+count)=0;

     for (count=0; count<rows; count++)			    /* for every data item */
	  *(*output+case_index[count])+=data[count];	    /* keep running total */

     for (count=0; count<cases; count++)		    /* for every data item */
	  *(*output+count)/=hits[count];		    /* divide running total by count */
}
