/*---------------------------------------------------------------------------------------------------------------------------------
   EPA - Region II, ESD
  ------------------------------------------------------------------------------
   Program: mode()
   Purpose: find the modal value for numeric data
  ------------------------------------------------------------------------------
   Usage:   see function
  ------------------------------------------------------------------------------
   Calls:   nothing
   Calling: general function
  ------------------------------------------------------------------------------
   Notes:   statistical functions some simple and some derived from:
William H. Press, Brian P. Flannery, Saul A. Teukolsky and William T. Vettering
    _NUMERICAL RECIPIES in C: The Art of Scientific Computing_
    Cambridge University Press, 1988
            This book is useful, but contains problems--most notably incorrect
	    subscripting of arrays (C is 0 to N-1; they use 1 to N).
  ------------------------------------------------------------------------------
   History:
     Author/Site:    Date:       Event:
   Alan Brenner      Jan 94      original coding
     alan_brenner@unixmail.edi.epa.gov
   Alan Brenner      Jan 95      added case based statistics
                                 added debuging
  ------------------------------------------------------------------------------
   Disclaimer:
     Although this program has been used by the U. S. Environmental Protection
   Agency, no warranty, expressed or implied, is made by the EPA as to the
   accurarcy and functioning of the program and related program material nor
   shall the fact of distribution constitute any such warranty, and no
   responsibility is assumed by the EPA in connection herewith.
---------------------------------------------------------------------------------------------------------------------------------*/

#include "stats.h"

double mode(rows, data)
  long rows;						    /* number of data items */
  double *data;						    /* data array */
{
     long i,j;						    /* for loop counters */
     long *cntr;					    /* the number of occurance for each unique item */
     long *uneke;					    /* index list of unique items in data[] */
     long unique=1;					    /* number of unique items */
     char swtch;					    /* current item unique true/false switch */

     if (debug) (void) fprintf(stderr,"mode(%d,%p)\n",rows,data);

     if ((cntr=calloc((unsigned int) rows,sizeof(long)))==NULL)
          error(-1,"Cannot allocate memory in mode.");
     if ((uneke=calloc((unsigned int) rows,sizeof(long)))==NULL)
          error(-2,"Cannot allocate memory in mode.");

     cntr[0]=1;						    /* first value is always unique */
     uneke[0]=0;

     for (i=1; i<rows; i++)				    /* for every other possible point */
     {
	  swtch=0;
	  for (j=0; j<unique; j++)			    /* check the current list unique of unique points */
	  {
	       if (data[i]==data[uneke[j]])		    /* if point is equal to member of unique set */
	       {
		    cntr[j]++;				    /* increment unique counter */
		    swtch=1;				    /* set swtich for not a new unique value */
	       }
	  }
	  if (swtch)					    /* unique new value */
	  {
	       unique++;				    /* increment total number of unique values */
	       uneke[unique]=i;				    /* copy value to unique list */
	       cntr[unique]=1;				    /* set unique count to one */
	  }
     }

     if (unique==1)					    /* all values are the same */
	  return(data[uneke[0]]);

     j=0;						    /* set the possible output to the first value */
     swtch=cntr[0];
     
     for (i=1; i<unique; i++)				    /* for each other uneke value */
     {
	  if (swtch<cntr[i])				    /* if the total number of occurances > current */
	  {
	       swtch=cntr[i];				    /* keep the new occurance */
	       j=i;					    /* keep the new data value */
	  }
     }

     return(data[uneke[j]]);				    /* output the (single) modal data value */
}
