/*---------------------------------------------------------------------------------------------------------------------------------
   EPA - Region II, ESD
  ------------------------------------------------------------------------------
   Program: skew()
   Purpose: find skewness
  ------------------------------------------------------------------------------
   Usage:   see function
  ------------------------------------------------------------------------------
   Calls:   nothing
   Calling: general function
  ------------------------------------------------------------------------------
   Notes:   statistical functions some simple and some derived from:
William H. Press, Brian P. Flannery, Saul A. Teukolsky and William T. Vettering
    _NUMERICAL RECIPIES in C: The Art of Scientific Computing_
    Cambridge University Press, 1988
            This book is useful, but contains problems--most notably incorrect
	    subscripting of arrays (C is 0 to N-1; they use 1 to N).
  ------------------------------------------------------------------------------
   History:
     Author/Site:    Date:       Event:
   Alan Brenner      Jan 94      original coding
     alan_brenner@unixmail.edi.epa.gov
   Alan Brenner      Jan 95      added case based statistics
                                 added debuging
  ------------------------------------------------------------------------------
   Disclaimer:
     Although this program has been used by the U. S. Environmental Protection
   Agency, no warranty, expressed or implied, is made by the EPA as to the
   accurarcy and functioning of the program and related program material nor
   shall the fact of distribution constitute any such warranty, and no
   responsibility is assumed by the EPA in connection herewith.
---------------------------------------------------------------------------------------------------------------------------------*/

#include "stats.h"

double skew(rows, ave, var, data)
  long rows;						    /* number of items in data[] */
  double ave;						    /* mean of values in data[] */
  double var;						    /* variance of values in data[] */
  double *data;						    /* data */
{
     long count;					    /* for loop counter */
     double skew=0.0;					    /* summartion of cubes of differences from the mean */

     if (debug) (void) fprintf(stderr,"skew(%d,%f,%f,%p)\n",rows,ave,var,data);
     if (debug>6)
     {
	  for (count=0; count<rows; count++)
	       (void) fprintf(stderr,"%10.0d = %f\n",count,data[count]);
     }

     if (rows<2)
     {
	  (void) fprintf(stderr,"There must be more than one data value to calculate skewness.\n");
	  (void) fprintf(stderr,"Returning 0 as NODATA.\n");
	  return(0.0);
     }
     if (var<0)
     {
	  (void) fprintf(stderr,"Garbage input for variance--it is less than zero.\n");
	  (void) fprintf(stderr,"Returning 0 as NODATA.\n");
	  return(0.0);
     }
     if (var==0)
     {
	  (void) fprintf(stderr,"Skewness cannot be calculated when variance is zero.\n");
	  (void) fprintf(stderr,"Returning 0 as NODATA.\n");
	  return(0.0);
     }

     if (debug>3) (void) fprintf(stderr,"Calculation summation of cubes of differences from the mean.\n");
     for (count=0; count<rows; count++)			    /* calculate sum of cubes */
	  skew+=pow(data[count]-ave,3);

     if (debug>3) (void) fprintf(stderr,"Returning skewness.\n");
     return(skew/(rows*var*sqrt(var)));			    /* return skewness */
}
