/*---------------------------------------------------------------------------------------------------------------------------------
   EPA - Region II, ESD
  ------------------------------------------------------------------------------
   Program: svarc()
   Purpose: find the sample variance for cased data
  ------------------------------------------------------------------------------
   Usage:   see function
  ------------------------------------------------------------------------------
   Calls:   nothing
   Calling: general function
  ------------------------------------------------------------------------------
   Notes:   statistical functions some simple and some derived from:
William H. Press, Brian P. Flannery, Saul A. Teukolsky and William T. Vettering
    _NUMERICAL RECIPIES in C: The Art of Scientific Computing_
    Cambridge University Press, 1988
            This book is useful, but contains problems--most notably incorrect
	    subscripting of arrays (C is 0 to N-1; they use 1 to N).
  ------------------------------------------------------------------------------
   History:
     Author/Site:    Date:       Event:
   Alan Brenner      Jan 94      original coding
     alan_brenner@unixmail.edi.epa.gov
   Alan Brenner      Jan 95      added case based statistics
                                 added debuging
  ------------------------------------------------------------------------------
   Disclaimer:
     Although this program has been used by the U. S. Environmental Protection
   Agency, no warranty, expressed or implied, is made by the EPA as to the
   accurarcy and functioning of the program and related program material nor
   shall the fact of distribution constitute any such warranty, and no
   responsibility is assumed by the EPA in connection herewith.
---------------------------------------------------------------------------------------------------------------------------------*/

#include "stats.h"

void svarc(rows, aves, data, cases, case_index, output)
  long rows;						    /* number of data items in data[], and case_index[] */
  long cases;						    /* number of unique values in case_index[] */
  long *case_index;					    /* index of data that holds which case each data value goes with */
  double *aves;						    /* the mean of each case in data[] */
  double *data;						    /* the actual data */
  double **output;					    /* address of an array for variance */
{
     long count,counter;				    /* for loop counters */
     long items;					    /* number of items in the current case */
     double *casedata;					    /* space for one cases' data */

     if (debug) (void) fprintf(stderr,"svarc(%d,%p,%p,%d,%p,%p)\n",rows,aves,data,cases,case_index,output);

     if ((casedata=calloc(rows,sizeof(double)))==NULL)	    /* space for case's data */
	  error(2,"Unable to allocate memory in function svarc.");

     for (count=0; count<cases; count++)		    /* for each case */
     {
	  for (items=counter=0; counter<rows; counter++)    /* find the values for each case */
	  {
	       if (case_index[counter]==count)		    /* this value is part of this case? */
	       {
		    casedata[items]=data[counter];	    /* keep data value */
		    items++;				    /* increment count */
	       }
	  }

	  if (items==1)					    /* if only one data item */
	       *(*output+count)=0.0;			    /* the variance is undefined */
	  else						    /* find & record the variance */
	       *(*output+count)=svar(items,aves[count],casedata);
     }
}
