/*------------------------------------------------------------------------------
   EPA - Region II, ESD
  ------------------------------------------------------------------------------
   Program: inforeselect
   Purpose: edit only reselected records
  ------------------------------------------------------------------------------
   Usage:   
  ------------------------------------------------------------------------------
   Calls:   once compiled, nothing
   Calling: used in tools using INFO via Todd Stellhorn's Infolib.
  ------------------------------------------------------------------------------
   Notes:   
  ------------------------------------------------------------------------------
   History:
     Author/Site:    Date:       Event:
   Alan Brenner    31 Jul 95   original coding
     Alan_Brenner@unixmail.edi.epa.gov
  ------------------------------------------------------------------------------
   Disclaimer:
     Although this program has been used by the U. S. Environmental Protection
   Agency, no warranty, expressed or implied, is made by the EPA as to the
   accurarcy and functioning of the program and related program material nor
   shall the fact of distribution constitute any such warranty, and no
   responsibility is assumed by the EPA in connection herewith.
------------------------------------------------------------------------------*/
#include "inforeselect.h"
#include "utils.h"

unsigned long InfoGetReselects(input, output)
  char *input;
  InfoReselectStruct **output;
{
     unsigned long records=1;		/* number of records in output array */

     if (debug)
	  (void) fprintf(stderr,"InfoGetReselects(%p,%p)\n",input,*output);

     if ((*output=calloc(2,(size_t) sizeof(InfoReselectStruct)))==NULL)
	  error(1,"Unable to allocate memory for record list.");
     (*output)[0].recnum=1;		/* space for current record counter */

     (*output)[records].recnum=atol(input); /* then get the first number */
     (*output)[records].range=0;	/* first number is exact match */

     while ((input=strpbrk(input,",-"))!=NULL)
     {					/* get space and the number */
	  records++;			/* increment the number of records */
	  if ((*output=realloc(*output,sizeof(InfoReselectStruct)))==NULL)
	       error(1,"Unable to reallocate space for record list.");
	  if (*input==',')		/* if it is a comma, range = false */
	       (*output)[records].range=0;
	  else				/* if it is a dash, range = true */
	       (*output)[records].range=1;
	  input++;			/* point to number */
	  (*output)[records].recnum=atol(input);

	  if ((*output)[records-1].recnum>=(*output)[records].recnum)
	       error(1,"Invalid (decreasing) ordering in record number range");
     }

     return(records);			/* return size of the output array */
}

/*============================================================================*/
char InfoCheckReselect(record, selects)
  unsigned long record;			/* current, possible input record */
  InfoReselectStruct *selects;		/* array of used records */
{
     unsigned long entry;		/* short for the value in 0.recnum */

     if (debug)
	  (void) fprintf(stderr,"InfoCheckReselect(%d,%p)\n",record,selects);

     if (selects==NULL)			/* no reselects->always use record */
	  return(1);

     entry=selects[0].recnum;
     if (selects[entry].range)		/* look within rangle of records */
     {					/* is record within the range */
	  if (selects[entry-1].recnum<record && record<=selects[entry].recnum)
	       return(1);		/* return true */
     }
     else				/* range of values */
     {					/* if the record matches */
	  if (selects[entry].recnum==record)
	       return(1);		/* return true */
     }
     return(0);				/* return false */
}

/*============================================================================*/
char InfoMoreReselects(record, last, selects)
  unsigned long record;
  unsigned long last;
  InfoReselectStruct **selects;
{
     if (debug)
	  (void) fprintf(stderr,"InfoCheckReselect(%d,%d,%p)\n",
			 record,last,selects);

     if (*selects==NULL)		/* no reselects->always say more */
	  return(1);

     if (record==(*selects)[(*selects)[0].recnum].range)
     {					/* if a specifically named record */
	  (*selects)[0].recnum++;	/* point to the next one */
	  if ((*selects)[0].recnum>last) /* if after the last used */
	       return(0);		/* return no more records */
     }
     return(1);
}
